pub fn execute_trap_handler(trap_cmd: &str, shell_state: &mut ShellState) -> i32Expand description
Execute a trap handler command Note: Signal masking during trap execution will be added in a future update
pub fn execute_trap_handler(trap_cmd: &str, shell_state: &mut ShellState) -> i32Execute a trap handler command Note: Signal masking during trap execution will be added in a future update