Module executor

Module executor 

Functions

execute
Evaluate an AST node within the provided shell state and return its exit code.
execute_trap_handler
Execute a trap handler command Note: Signal masking during trap execution will be added in a future update
expand_variables_in_string
Expands shell-style variables, command substitutions, arithmetic expressions, and backtick substitutions inside a string.