Functions
- execute
- Evaluate an AST node within the provided shell state and return its exit code.
- execute_
trap_ handler - Execute a trap handler command Note: Signal masking during trap execution will be added in a future update
- expand_
variables_ in_ string - Expands shell-style variables, command substitutions, arithmetic expressions, and backtick substitutions inside a string.